@@ -25,6 +25,7 @@
 #include <telepathy-glib/base-connection.h>
 #include <telepathy-glib/contacts-mixin.h>
 #include <telepathy-glib/presence-mixin.h>
+#include <telepathy-glib/simple-password-manager.h>
 
 #include <libpurple/account.h>
 #include <libpurple/prpl.h>
@@ -35,18 +36,6 @@
 
 G_BEGIN_DECLS
 
-/* Must be in the same order as list_handle_strings in connection.c */
-typedef enum
-{
-    HAZE_LIST_HANDLE_SUBSCRIBE = 1,
-    HAZE_LIST_HANDLE_PUBLISH,
-#if 0
-    HAZE_LIST_HANDLE_HIDE,
-    HAZE_LIST_HANDLE_ALLOW,
-    HAZE_LIST_HANDLE_DENY
-#endif
-} HazeListHandle;
-
 typedef struct _HazeConnection HazeConnection;
 typedef struct _HazeConnectionPrivate HazeConnectionPrivate;
 typedef struct _HazeConnectionClass HazeConnectionClass;
@@ -66,6 +55,7 @@ struct _HazeConnection {
     HazeContactList *contact_list;
     HazeImChannelFactory *im_factory;
     HazeMediaManager *media_manager;
+    TpSimplePasswordManager *password_manager;
 
     TpContactsMixin contacts;
     TpPresenceMixin presence;
@@ -116,6 +106,9 @@ GType haze_connection_get_type (void);
 
 const gchar *haze_get_fallback_group (void);
 
+const gchar **haze_connection_get_implemented_interfaces (void);
+const gchar **haze_connection_get_guaranteed_interfaces (void);
+
 G_END_DECLS
 
 #endif /* #ifndef __HAZE_CONNECTION_H__*/
